var init = (function (GLib, GObject, clutter10, st1) {
  'use strict';

  function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

  var GLib__default = /*#__PURE__*/_interopDefaultLegacy(GLib);
  var GObject__default = /*#__PURE__*/_interopDefaultLegacy(GObject);

  function toInteger(dirtyNumber) {
    if (dirtyNumber === null || dirtyNumber === true || dirtyNumber === false) {
      return NaN;
    }
    var number = Number(dirtyNumber);
    if (isNaN(number)) {
      return number;
    }
    return number < 0 ? Math.ceil(number) : Math.floor(number);
  }

  function requiredArgs(required, args) {
    if (args.length < required) {
      throw new TypeError(required + ' argument' + (required > 1 ? 's' : '') + ' required, but only ' + args.length + ' present');
    }
  }

  function _typeof$1(obj) { "@babel/helpers - typeof"; if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof$1 = function _typeof(obj) { return typeof obj; }; } else { _typeof$1 = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof$1(obj); }
  function toDate(argument) {
    requiredArgs(1, arguments);
    var argStr = Object.prototype.toString.call(argument);
    if (argument instanceof Date || _typeof$1(argument) === 'object' && argStr === '[object Date]') {
      return new Date(argument.getTime());
    } else if (typeof argument === 'number' || argStr === '[object Number]') {
      return new Date(argument);
    } else {
      if ((typeof argument === 'string' || argStr === '[object String]') && typeof console !== 'undefined') {
        console.warn("Starting with v2.0.0-beta.1 date-fns doesn't accept strings as date arguments. Please use `parseISO` to parse strings. See: https://github.com/date-fns/date-fns/blob/master/docs/upgradeGuide.md#string-arguments");
        console.warn(new Error().stack);
      }
      return new Date(NaN);
    }
  }

  function addDays(dirtyDate, dirtyAmount) {
    requiredArgs(2, arguments);
    var date = toDate(dirtyDate);
    var amount = toInteger(dirtyAmount);
    if (isNaN(amount)) {
      return new Date(NaN);
    }
    if (!amount) {
      return date;
    }
    date.setDate(date.getDate() + amount);
    return date;
  }

  function addMonths(dirtyDate, dirtyAmount) {
    requiredArgs(2, arguments);
    var date = toDate(dirtyDate);
    var amount = toInteger(dirtyAmount);
    if (isNaN(amount)) {
      return new Date(NaN);
    }
    if (!amount) {
      return date;
    }
    var dayOfMonth = date.getDate();
    var endOfDesiredMonth = new Date(date.getTime());
    endOfDesiredMonth.setMonth(date.getMonth() + amount + 1, 0);
    var daysInMonth = endOfDesiredMonth.getDate();
    if (dayOfMonth >= daysInMonth) {
      return endOfDesiredMonth;
    } else {
      date.setFullYear(endOfDesiredMonth.getFullYear(), endOfDesiredMonth.getMonth(), dayOfMonth);
      return date;
    }
  }

  function _typeof(obj) { "@babel/helpers - typeof"; if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }
  function add(dirtyDate, duration) {
    requiredArgs(2, arguments);
    if (!duration || _typeof(duration) !== 'object') return new Date(NaN);
    var years = duration.years ? toInteger(duration.years) : 0;
    var months = duration.months ? toInteger(duration.months) : 0;
    var weeks = duration.weeks ? toInteger(duration.weeks) : 0;
    var days = duration.days ? toInteger(duration.days) : 0;
    var hours = duration.hours ? toInteger(duration.hours) : 0;
    var minutes = duration.minutes ? toInteger(duration.minutes) : 0;
    var seconds = duration.seconds ? toInteger(duration.seconds) : 0;
    var date = toDate(dirtyDate);
    var dateWithMonths = months || years ? addMonths(date, months + years * 12) : date;
    var dateWithDays = days || weeks ? addDays(dateWithMonths, days + weeks * 7) : dateWithMonths;
    var minutesToAdd = minutes + hours * 60;
    var secondsToAdd = seconds + minutesToAdd * 60;
    var msToAdd = secondsToAdd * 1000;
    var finalDate = new Date(dateWithDays.getTime() + msToAdd);
    return finalDate;
  }

  function getTimezoneOffsetInMilliseconds(date) {
    var utcDate = new Date(Date.UTC(date.getFullYear(), date.getMonth(), date.getDate(), date.getHours(), date.getMinutes(), date.getSeconds(), date.getMilliseconds()));
    utcDate.setUTCFullYear(date.getFullYear());
    return date.getTime() - utcDate.getTime();
  }

  function startOfDay(dirtyDate) {
    requiredArgs(1, arguments);
    var date = toDate(dirtyDate);
    date.setHours(0, 0, 0, 0);
    return date;
  }

  var MILLISECONDS_IN_DAY = 86400000;
  function differenceInCalendarDays(dirtyDateLeft, dirtyDateRight) {
    requiredArgs(2, arguments);
    var startOfDayLeft = startOfDay(dirtyDateLeft);
    var startOfDayRight = startOfDay(dirtyDateRight);
    var timestampLeft = startOfDayLeft.getTime() - getTimezoneOffsetInMilliseconds(startOfDayLeft);
    var timestampRight = startOfDayRight.getTime() - getTimezoneOffsetInMilliseconds(startOfDayRight);
    return Math.round((timestampLeft - timestampRight) / MILLISECONDS_IN_DAY);
  }

  function compareAsc(dirtyDateLeft, dirtyDateRight) {
    requiredArgs(2, arguments);
    var dateLeft = toDate(dirtyDateLeft);
    var dateRight = toDate(dirtyDateRight);
    var diff = dateLeft.getTime() - dateRight.getTime();
    if (diff < 0) {
      return -1;
    } else if (diff > 0) {
      return 1;
    } else {
      return diff;
    }
  }

  var millisecondsInMinute = 60000;
  var millisecondsInHour = 3600000;

  function differenceInCalendarMonths(dirtyDateLeft, dirtyDateRight) {
    requiredArgs(2, arguments);
    var dateLeft = toDate(dirtyDateLeft);
    var dateRight = toDate(dirtyDateRight);
    var yearDiff = dateLeft.getFullYear() - dateRight.getFullYear();
    var monthDiff = dateLeft.getMonth() - dateRight.getMonth();
    return yearDiff * 12 + monthDiff;
  }

  function differenceInCalendarYears(dirtyDateLeft, dirtyDateRight) {
    requiredArgs(2, arguments);
    var dateLeft = toDate(dirtyDateLeft);
    var dateRight = toDate(dirtyDateRight);
    return dateLeft.getFullYear() - dateRight.getFullYear();
  }

  function compareLocalAsc(dateLeft, dateRight) {
    var diff = dateLeft.getFullYear() - dateRight.getFullYear() || dateLeft.getMonth() - dateRight.getMonth() || dateLeft.getDate() - dateRight.getDate() || dateLeft.getHours() - dateRight.getHours() || dateLeft.getMinutes() - dateRight.getMinutes() || dateLeft.getSeconds() - dateRight.getSeconds() || dateLeft.getMilliseconds() - dateRight.getMilliseconds();
    if (diff < 0) {
      return -1;
    } else if (diff > 0) {
      return 1;
    } else {
      return diff;
    }
  }
  function differenceInDays(dirtyDateLeft, dirtyDateRight) {
    requiredArgs(2, arguments);
    var dateLeft = toDate(dirtyDateLeft);
    var dateRight = toDate(dirtyDateRight);
    var sign = compareLocalAsc(dateLeft, dateRight);
    var difference = Math.abs(differenceInCalendarDays(dateLeft, dateRight));
    dateLeft.setDate(dateLeft.getDate() - sign * difference);
    var isLastDayNotFull = Number(compareLocalAsc(dateLeft, dateRight) === -sign);
    var result = sign * (difference - isLastDayNotFull);
    return result === 0 ? 0 : result;
  }

  function differenceInMilliseconds(dateLeft, dateRight) {
    requiredArgs(2, arguments);
    return toDate(dateLeft).getTime() - toDate(dateRight).getTime();
  }

  var roundingMap = {
    ceil: Math.ceil,
    round: Math.round,
    floor: Math.floor,
    trunc: function trunc(value) {
      return value < 0 ? Math.ceil(value) : Math.floor(value);
    }
  };
  var defaultRoundingMethod = 'trunc';
  function getRoundingMethod(method) {
    return method ? roundingMap[method] : roundingMap[defaultRoundingMethod];
  }

  function differenceInHours(dateLeft, dateRight, options) {
    requiredArgs(2, arguments);
    var diff = differenceInMilliseconds(dateLeft, dateRight) / millisecondsInHour;
    return getRoundingMethod(options === null || options === void 0 ? void 0 : options.roundingMethod)(diff);
  }

  function differenceInMinutes(dateLeft, dateRight, options) {
    requiredArgs(2, arguments);
    var diff = differenceInMilliseconds(dateLeft, dateRight) / millisecondsInMinute;
    return getRoundingMethod(options === null || options === void 0 ? void 0 : options.roundingMethod)(diff);
  }

  function endOfDay(dirtyDate) {
    requiredArgs(1, arguments);
    var date = toDate(dirtyDate);
    date.setHours(23, 59, 59, 999);
    return date;
  }

  function endOfMonth(dirtyDate) {
    requiredArgs(1, arguments);
    var date = toDate(dirtyDate);
    var month = date.getMonth();
    date.setFullYear(date.getFullYear(), month + 1, 0);
    date.setHours(23, 59, 59, 999);
    return date;
  }

  function isLastDayOfMonth(dirtyDate) {
    requiredArgs(1, arguments);
    var date = toDate(dirtyDate);
    return endOfDay(date).getTime() === endOfMonth(date).getTime();
  }

  function differenceInMonths(dirtyDateLeft, dirtyDateRight) {
    requiredArgs(2, arguments);
    var dateLeft = toDate(dirtyDateLeft);
    var dateRight = toDate(dirtyDateRight);
    var sign = compareAsc(dateLeft, dateRight);
    var difference = Math.abs(differenceInCalendarMonths(dateLeft, dateRight));
    var result;
    if (difference < 1) {
      result = 0;
    } else {
      if (dateLeft.getMonth() === 1 && dateLeft.getDate() > 27) {
        dateLeft.setDate(30);
      }
      dateLeft.setMonth(dateLeft.getMonth() - sign * difference);
      var isLastMonthNotFull = compareAsc(dateLeft, dateRight) === -sign;
      if (isLastDayOfMonth(toDate(dirtyDateLeft)) && difference === 1 && compareAsc(dirtyDateLeft, dateRight) === 1) {
        isLastMonthNotFull = false;
      }
      result = sign * (difference - Number(isLastMonthNotFull));
    }
    return result === 0 ? 0 : result;
  }

  function differenceInSeconds(dateLeft, dateRight, options) {
    requiredArgs(2, arguments);
    var diff = differenceInMilliseconds(dateLeft, dateRight) / 1000;
    return getRoundingMethod(options === null || options === void 0 ? void 0 : options.roundingMethod)(diff);
  }

  function differenceInWeeks(dateLeft, dateRight, options) {
    requiredArgs(2, arguments);
    var diff = differenceInDays(dateLeft, dateRight) / 7;
    return getRoundingMethod(options === null || options === void 0 ? void 0 : options.roundingMethod)(diff);
  }

  function differenceInYears(dirtyDateLeft, dirtyDateRight) {
    requiredArgs(2, arguments);
    var dateLeft = toDate(dirtyDateLeft);
    var dateRight = toDate(dirtyDateRight);
    var sign = compareAsc(dateLeft, dateRight);
    var difference = Math.abs(differenceInCalendarYears(dateLeft, dateRight));
    dateLeft.setFullYear(1584);
    dateRight.setFullYear(1584);
    var isLastYearNotFull = compareAsc(dateLeft, dateRight) === -sign;
    var result = sign * (difference - Number(isLastYearNotFull));
    return result === 0 ? 0 : result;
  }

  // Unpack the settings
  function unpackSettings(settings) {
      return {
          formatString: settings.get_string("format-string"),
          birthYear: settings.get_int("birth-year"),
          birthMonth: settings.get_int("birth-month"),
          birthDay: settings.get_int("birth-day"),
          lifeExpectancy: settings.get_int("life-expectancy"),
          extensionPosition: settings.get_enum("extension-position"),
          extensionIndex: settings.get_int("extension-index"),
      };
  }

  /* eslint-disable @typescript-eslint/no-explicit-any */
  const Main = imports.ui.main;
  const Mainloop = imports.mainloop;
  const { Button } = imports.ui.panelMenu;
  const { getSettings, initTranslations } = imports.misc.extensionUtils;
  const _ = imports.misc.extensionUtils.gettext;
  // Indicator
  const Indicator = GObject__default["default"].registerClass(class Indicator extends Button {
      // Settings
      #settings;
      // The date of passing away
      #end;
      // Label
      #label;
      // Construct the indicator
      constructor() {
          // Initialize the button
          super(
          // Menu alignment
          0, 
          // Name of the button
          _("Memento Mori Extension"), 
          // Don't create the menu?
          true);
          // Obtain and unpack the settings
          this.#settings = getSettings();
          const settingsValues = unpackSettings(this.#settings);
          // Save the end date
          this.#end = Indicator.getEndDate(settingsValues);
          // Add the label
          this.#label = new st1.Label({
              text: Indicator.getText(settingsValues, this.#end),
              yAlign: clutter10.ActorAlign.CENTER,
          });
          this.add_child(this.#label);
      }
      // Update the indicator
      update() {
          // Unpack the settings
          const settingsValues = unpackSettings(this.#settings);
          // Update the fields
          this.#end = Indicator.getEndDate(settingsValues);
          this.#label.text = Indicator.getText(settingsValues, this.#end);
      }
      // Get the text for the label
      static getText(settingsValues, end) {
          // Get the current date
          const start = new Date();
          // Get each part of the time stamp
          const years = Math.max(0, differenceInYears(end, start));
          const months = Math.max(0, differenceInMonths(end, start));
          const weeks = Math.max(0, differenceInWeeks(end, start));
          const days = Math.max(0, differenceInDays(end, start));
          const hours = Math.max(0, differenceInHours(end, start));
          const minutes = Math.max(0, differenceInMinutes(end, start));
          const seconds = Math.max(0, differenceInSeconds(end, start));
          // Return the formatted string
          return settingsValues.formatString
              .replace("${y}", `${years}`)
              .replace("${M}", `${months}`)
              .replace("${w}", `${weeks}`)
              .replace("${d}", `${days}`)
              .replace("${h}", `${hours}`)
              .replace("${m}", `${minutes}`)
              .replace("${s}", `${seconds}`);
      }
      // Get the date of expected passing away
      static getEndDate(settingsValues) {
          // Construct and return the date
          return add(new Date(settingsValues.birthYear, settingsValues.birthMonth - 1, settingsValues.birthDay), {
              years: settingsValues.lifeExpectancy,
          });
      }
  });
  // Extension
  class Extension {
      // UUID of the extension
      #uuid;
      // Settings
      #settings;
      // Signals
      #signals;
      // Extension position
      #extensionPosition;
      // Extension index
      #extensionIndex;
      // Indicator
      #indicator;
      // Timeout source
      #timeout = 0;
      // Construct the extension
      constructor(uuid) {
          this.#uuid = uuid;
          this.#settings = getSettings();
          this.#signals = [];
          this.#extensionPosition = Extension.getExtensionPosition(this.#settings);
          this.#extensionIndex = this.#settings.get_int("extension-index");
      }
      // Enable the extension
      enable() {
          // Add the indicator
          this.addIndicator();
          // Update the indicator on any change to counter settings
          for (const key of [
              "format-string",
              "birth-year",
              "birth-month",
              "birth-day",
              "life-expectancy",
          ]) {
              const handlerId = this.#settings.connect(`changed::${key}`, () => {
                  this.#indicator.update();
              });
              this.#signals.push(handlerId);
          }
          // Recreate the indicator in case the extension position is changed
          {
              const handlerId = this.#settings.connect("changed::extension-position", () => {
                  this.#extensionPosition = Extension.getExtensionPosition(this.#settings);
                  this.removeIndicator();
                  this.addIndicator();
              });
              this.#signals.push(handlerId);
          }
          // Recreate the indicator in case the extension index is changed
          {
              const handlerId = this.#settings.connect("changed::extension-index", () => {
                  this.#extensionIndex =
                      this.#settings.get_int("extension-index");
                  this.removeIndicator();
                  this.addIndicator();
              });
              this.#signals.push(handlerId);
          }
      }
      // Add the indicator to the panel
      addIndicator() {
          // Initialize a new indicator
          this.#indicator = new Indicator();
          // Add the indicator to the panel
          Main.panel.addToStatusArea(
          // Role
          this.#uuid, 
          // Indicator
          this.#indicator, 
          // Index
          this.#extensionIndex, 
          // Position
          this.#extensionPosition);
          // Update the indicator every second
          this.#timeout = Mainloop.timeout_add_seconds(1, () => {
              // Update the indicator
              this.#indicator.update();
              // Leave the source in the main loop
              return GLib__default["default"].SOURCE_CONTINUE;
          });
      }
      // Remove the indicator from the panel
      removeIndicator() {
          // If the timeout is set
          if (this.#timeout) {
              // Remove the timeout
              Mainloop.source_remove(this.#timeout);
          }
          // If the indicator exists
          if (this.#indicator) {
              // Free the GObject
              this.#indicator.destroy();
              // Stop tracing it
              this.#indicator = null;
          }
      }
      // Disable the extension
      disable() {
          // If there are connected signals, disconnect them
          for (const handlerId of this.#signals) {
              this.#settings.disconnect(handlerId);
          }
          this.#signals = [];
          // Remove the indicator
          this.removeIndicator();
      }
      // Get the extension position from the settings
      static getExtensionPosition(settings) {
          switch (settings.get_enum("extension-position")) {
              case 0:
                  return "left";
              case 1:
                  return "center";
              case 2:
                  return "right";
              default:
                  return "right";
          }
      }
  }
  // Initialize the extension
  function init(meta) {
      // Initialize translations
      initTranslations(meta.uuid);
      // Construct and return the extension
      return new Extension(meta.uuid);
  }

  return init;

})(imports.gi.GLib, imports.gi.GObject, imports.gi.Clutter, imports.gi.St);
