var prefs = (function (Adw, GLib, Gio, Gtk) {
    'use strict';

    function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

    var Adw__default = /*#__PURE__*/_interopDefaultLegacy(Adw);
    var GLib__default = /*#__PURE__*/_interopDefaultLegacy(GLib);
    var Gio__default = /*#__PURE__*/_interopDefaultLegacy(Gio);
    var Gtk__default = /*#__PURE__*/_interopDefaultLegacy(Gtk);

    function dedent(templ) {
        var values = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            values[_i - 1] = arguments[_i];
        }
        var strings = Array.from(typeof templ === 'string' ? [templ] : templ);
        strings[strings.length - 1] = strings[strings.length - 1].replace(/\r?\n([\t ]*)$/, '');
        var indentLengths = strings.reduce(function (arr, str) {
            var matches = str.match(/\n([\t ]+|(?!\s).)/g);
            if (matches) {
                return arr.concat(matches.map(function (match) { var _a, _b; return (_b = (_a = match.match(/[\t ]/g)) === null || _a === void 0 ? void 0 : _a.length) !== null && _b !== void 0 ? _b : 0; }));
            }
            return arr;
        }, []);
        if (indentLengths.length) {
            var pattern_1 = new RegExp("\n[\t ]{" + Math.min.apply(Math, indentLengths) + "}", 'g');
            strings = strings.map(function (str) { return str.replace(pattern_1, '\n'); });
        }
        strings[0] = strings[0].replace(/^\r?\n/, '');
        var string = strings[0];
        values.forEach(function (value, i) {
            var endentations = string.match(/(?:^|\n)( *)$/);
            var endentation = endentations ? endentations[1] : '';
            var indentedValue = value;
            if (typeof value === 'string' && value.includes('\n')) {
                indentedValue = String(value)
                    .split('\n')
                    .map(function (str, i) {
                    return i === 0 ? str : "" + endentation + str;
                })
                    .join('\n');
            }
            string += indentedValue + strings[i + 1];
        });
        return string;
    }

    // Unpack the settings
    function unpackSettings(settings) {
        return {
            formatString: settings.get_string("format-string"),
            birthYear: settings.get_int("birth-year"),
            birthMonth: settings.get_int("birth-month"),
            birthDay: settings.get_int("birth-day"),
            lifeExpectancy: settings.get_int("life-expectancy"),
            extensionPosition: settings.get_enum("extension-position"),
            extensionIndex: settings.get_int("extension-index"),
        };
    }
    // Format the birthday as a string
    function formatBirthday(year, month, day) {
        // Convert the month and the day to strings
        const monthString = month < 10 ? `0${month}` : `${month}`;
        const dayString = day < 10 ? `0${day}` : `${day}`;
        // Return the formatted date
        return `${year}/${monthString}/${dayString}`;
    }

    /* eslint-disable @typescript-eslint/no-unsafe-call */
    const { getSettings, initTranslations } = imports.misc.extensionUtils;
    const _ = imports.misc.extensionUtils.gettext;
    function fillPreferencesWindow(window) {
        // Unpack the settings
        const settings = getSettings();
        const settingsValues = unpackSettings(settings);
        // Add the page to the window
        const preferencesPage = new Adw__default["default"].PreferencesPage();
        window.add(preferencesPage);
        // Add the counter group of preferences to the page
        const counterGroup = new Adw__default["default"].PreferencesGroup({
            title: _("Counter"),
        });
        preferencesPage.add(counterGroup);
        // Add an action row for the format string
        const formatStringActionRow = new Adw__default["default"].ActionRow({
            title: _("Format string"),
            subtitle: _("Hover over the entry to get more info"),
        });
        counterGroup.add(formatStringActionRow);
        // Prepare an entry buffer for the next entry
        const formatStringEntryBuffer = new Gtk__default["default"].EntryBuffer();
        // Connect the entry buffer with the settings
        settings.bind("format-string", formatStringEntryBuffer, "text", Gio__default["default"].SettingsBindFlags.DEFAULT);
        // Add an entry for the last action row
        const formatStringEntry = new Gtk__default["default"].Entry({
            buffer: formatStringEntryBuffer,
            margin_top: 10,
            margin_bottom: 10,
            placeholder_text: "${w} weeks remaining",
            tooltip_text: dedent(_("Available modifiers for representing\n" +
                "total number of <blank> remaining:\n" +
                "  - ${y}: Years\n" +
                "  - ${M}: Months\n" +
                "  - ${w}: Weeks\n" +
                "  - ${d}: Days\n" +
                "  - ${h}: Hours\n" +
                "  - ${m}: Minutes\n" +
                "  - ${s}: Seconds")),
        });
        formatStringActionRow.add_suffix(formatStringEntry);
        formatStringActionRow.set_activatable_widget(formatStringEntry);
        // Add an action row for the birthday
        const birthdayActionRow = new Adw__default["default"].ActionRow({
            title: _("Birthday"),
            subtitle: _("Your birthday in the form YYYY/MM/DD"),
        });
        counterGroup.add(birthdayActionRow);
        // Add a menu button to the last action row
        const birthdayButton = new Gtk__default["default"].MenuButton({
            label: formatBirthday(settingsValues.birthYear, settingsValues.birthMonth, settingsValues.birthDay),
            margin_top: 10,
            margin_bottom: 10,
        });
        birthdayActionRow.add_suffix(birthdayButton);
        birthdayActionRow.set_activatable_widget(birthdayButton);
        // Add a popover menu for the last button
        const birthdayPopover = new Gtk__default["default"].Popover();
        birthdayButton.set_popover(birthdayPopover);
        // Add a calendar to the popover menu
        const birthdayCalendar = new Gtk__default["default"].Calendar();
        birthdayCalendar.select_day(GLib__default["default"].DateTime.new_local(settingsValues.birthYear, settingsValues.birthMonth, settingsValues.birthDay, 0, 0, 0));
        birthdayPopover.set_child(birthdayCalendar);
        // Synchronise the settings with the calendar
        function syncSettings(calendar) {
            // Unpack the values from the calendar
            const year = calendar.year;
            const month = calendar.month + 1;
            const day = calendar.day;
            // Update the label of the menu button
            birthdayButton.set_label(formatBirthday(year, month, day));
            // Update the settings
            settings.set_int("birth-year", year);
            settings.set_int("birth-month", month);
            settings.set_int("birth-day", day);
        }
        birthdayCalendar.connect("day-selected", (c) => syncSettings(c));
        birthdayCalendar.connect("prev-year", (c) => syncSettings(c));
        birthdayCalendar.connect("next-year", (c) => syncSettings(c));
        birthdayCalendar.connect("prev-month", (c) => syncSettings(c));
        birthdayCalendar.connect("next-month", (c) => syncSettings(c));
        // Add an action row for the life expectancy
        const lifeExpectancyActionRow = new Adw__default["default"].ActionRow({
            title: _("Life expectancy"),
            subtitle: _("Number of years you expect to live through"),
        });
        counterGroup.add(lifeExpectancyActionRow);
        // Add a spin button to the last action row
        const lifeExpectancySpinButton = new Gtk__default["default"].SpinButton({
            adjustment: new Gtk__default["default"].Adjustment({
                lower: 0,
                upper: 200,
                step_increment: 1,
            }),
            numeric: true,
            margin_top: 10,
            margin_bottom: 10,
        });
        lifeExpectancyActionRow.add_suffix(lifeExpectancySpinButton);
        lifeExpectancyActionRow.set_activatable_widget(lifeExpectancySpinButton);
        // Connect the spin button with the settings
        settings.bind("life-expectancy", lifeExpectancySpinButton, "value", Gio__default["default"].SettingsBindFlags.DEFAULT);
        // Add the appearance group of preferences to the page
        const appearanceGroup = new Adw__default["default"].PreferencesGroup({
            title: _("Appearance"),
        });
        preferencesPage.add(appearanceGroup);
        // Prepare a string list for the next combo row
        const extensionPositionOptions = new Gtk__default["default"].StringList();
        extensionPositionOptions.append(_("Left"));
        extensionPositionOptions.append(_("Center"));
        extensionPositionOptions.append(_("Right"));
        // Add a combo row for the extension position
        const extensionPositionComboRow = new Adw__default["default"].ComboRow({
            title: _("Extension position"),
            subtitle: _("Position of the extension in the panel"),
            model: extensionPositionOptions,
            selected: settingsValues.extensionPosition,
        });
        // Connect the combo row with the settings
        extensionPositionComboRow.connect("notify::selected", (cr) => {
            settings.set_enum("extension-position", cr.selected);
        });
        appearanceGroup.add(extensionPositionComboRow);
        // Add an action row for the extension index
        const extensionIndexActionRow = new Adw__default["default"].ActionRow({
            title: _("Extension index"),
            subtitle: _("Index of the extension in the panel"),
        });
        appearanceGroup.add(extensionIndexActionRow);
        // Add a spin button to the last action row
        const extensionIndexSpinButton = new Gtk__default["default"].SpinButton({
            adjustment: new Gtk__default["default"].Adjustment({
                lower: 0,
                upper: 10,
                step_increment: 1,
            }),
            numeric: true,
            margin_top: 10,
            margin_bottom: 10,
        });
        extensionIndexActionRow.add_suffix(extensionIndexSpinButton);
        extensionIndexActionRow.set_activatable_widget(extensionIndexSpinButton);
        // Connect the spin button with the settings
        settings.bind("extension-index", extensionIndexSpinButton, "value", Gio__default["default"].SettingsBindFlags.DEFAULT);
    }
    // Initialize the preferences
    function init(meta) {
        // Initialize translations
        initTranslations(meta.uuid);
    }
    var prefs = { init, fillPreferencesWindow };

    return prefs;

})(imports.gi.Adw, imports.gi.GLib, imports.gi.Gio, imports.gi.Gtk);
var init = prefs.init;
var fillPreferencesWindow = prefs.fillPreferencesWindow;
